;;; ====================================================================================================
;;; Big Horrible Hairy Tramp Section

;;; Key Concepts:
;;;  Connection Local Variables:                 https://www.gnu.org/software/emacs/manual/html_node/elisp/Connection-Local-Variables.html
;;;                                              https://www.gnu.org/software/tramp/#Running-shell-on-a-remote-host
;;;  Setting own connection related information: https://www.gnu.org/software/tramp/#Predefined-connection-information

;;; Testing Hosts:
;;;  rMBP, MacMini2: 	Mac OS X
;;;  hyper-m:		Solaris x86 / SunOS 5.10
;;;  aws:		Ubuntu 9.10 (karmic)
;;;  katolivadi: 	Linux Mint 18 Sarah
;;;  red-october: 	Linux Mint 18 Sarah
;;;  jlp:		Ubuntu 16.04.4 LTS (xenial)

;;; References:
;;;  http://www.gnu.org/software/emacs/manual/html_node/tramp/
;;;  https://www.gnu.org/software/emacs/manual/html_node/tramp/Predefined-connection-information.html
;;;  https://lists.gnu.org/archive/html/help-gnu-emacs/2015-08/msg00596.html
;;;  https://www.gnu.org/software/emacs/manual/html_node/tramp/Remote-processes.html
;;;  https://www.gnu.org/software/emacs/manual/html_node/elisp/Connection-Local-Variables.html
;;;  https://www.gnu.org/software/emacs/manual/html_node/tramp/Remote-processes.html
;;;  https://emacs.stackexchange.com/questions/32844/edit-via-tramp-ssh-fails-on-solaris-host
;;;  https://emacs.stackexchange.com/questions/12904/tramp-history-files
;;;  https://writequit.org/denver-emacs/presentations/2017-06-20-tramp.html

(require 'tramp)
(setq tramp-verbose 6)
(setq tramp-debug-buffer t)
;(setq tramp-password-prompt-regexp ".*[Pp]assword: *$")
;(setq tramp-shell-prompt-pattern "^[^;$#>]*[;$#>] *")
;(setq password-cache-expiry nil)

;;; tramp / shell history file took ages to iron out. still not
;;; completely sure how it works, because on some hosts like ubuntu,
;;; the tramp garbage did not go into the history file by default (ie
;;; no special settings). however the "exec env" lines were seen in
;;; the history file. but on solaris, it took a lot of work to ignore
;;; the tramp garbage but save my shell commands to the history file.

;;; tramp-histfile-override default value is "~/.tramp_history". both garbage and my legit commands go there
(setq tramp-histfile-override "/dev/null")        ;;; works to suppress all tramp garbage. but then, none of my shell commands get saved in history. solution: .bash_profile: export HISTFILE=/home/nico/.bash_history
;(setq tramp-histfile-override nil)               ;;; does not set HISTFILE at all, so each connection it defaults to the shell's default. this is a flexible setting, but tramp garbage still goes into history file
;(setq tramp-histfile-override "~/.bash_history") ;;; to make this work on solaris, set remote-shell to /usr/bin/bash (as above in tramp-connection-properties). remaining problem is tramp garbage now goes into ~/.bash_history

;;; customize ssh program
;;; I could never get this to work even after a few hours slogging... :-(
;;; in the end I just put /usr/local/bin in front of emacs' PATH
;;; (add-to-list 'tramp-connection-properties (list (regexp-quote "/ssh:aws:")            "tramp-login-program" "/usr/local/bin/ssh"))

(add-to-list 'tramp-connection-properties
	     (list
	      nil 				;;; nil matches all methods & hosts!
              "locale"				;;; ie tramp-methods tramp-locale
	      "LC_ALL=C"))

(add-to-list 'tramp-connection-properties
             (list "/.*:.*hyper-m:"		;;; all tramp on hyper-m should use /usr/bin/bash. this makes the tilde in tramp-histfile-override="~/.bash_history" work
                   "remote-shell"		;;; ie tramp-methods tramp-remote-shell
		   "/usr/bin/bash"))

(add-to-list 'tramp-connection-properties
             (list "/.*:.*hyper-m.wan:"		;;; all tramp on hyper-m should use /usr/bin/bash. this makes the tilde in tramp-histfile-override="~/.bash_history" work
                   "remote-shell"		;;; ie tramp-methods tramp-remote-shell
		   "/usr/bin/bash"))

;;; these didn't work :(
;; (add-to-list 'tramp-connection-properties
;;  	     (list ":hyper-m:"			;;; match default username (which is nico)
;;  		   "login-env"			;;; ie tramp-methods tramp-login-env
;; 		   (list
;; 		    (list "HISTFILE")
;; 		    (list "/home/nico/.bash_history_tramp-connection-properties_this-doesnt-work"))))

;; (add-to-list 'tramp-connection-properties
;;  	     (list ":nico@hyper-m:"		;;; match explicit username nico
;;  		   "login-env"			;;; ie tramp-methods tramp-login-env
;; 		   (list
;; 		    (list "HISTFILE")
;; 		    (list "/home/nico/.bash_history_tramp-connection-properties_this-doesnt-work"))))

;; (add-to-list 'tramp-connection-properties
;;  	     (list ":magento@jlp:"		;;; match explicit username magento
;;  		   "login-env"			;;; ie tramp-methods tramp-login-env
;; 		   (list
;; 		    (list "HISTFILE")
;; 		    (list "/home/magento/.bash_history_tramp-connection-properties_this-doesnt-work"))))

;;; these didn't work :(
;; (connection-local-set-profile-variables
;;  'nico-hyper-m-local-settings
;;  '((nico-test-variable . "/tmp/nico-test")))
;;
;; (connection-local-set-profiles
;;  '(:application 'tramp :protocol "ssh" :machine "hyper-m")
;;  'nico-hyper-m-local-settings)

(add-hook 'shell-mode-hook
	  '(lambda ()
	     (message "in shell mode hook, tramp-current-host = [%s]" tramp-current-host)
	     (message "in shell mode hook, (file-remote-p default-directory) = [%s]" (file-remote-p default-directory))
	     (message "in shell mode hook, comint-input-ring-file-name = [%s]" comint-input-ring-file-name)
	     ;;(if tramp-current-host
	     (if (file-remote-p default-directory)
		 (progn
		   ;; hack to load emacs history ring from remote shells
		   (message "emacs thinks this is a remote buffer")
		   (let ((history-file (concat "/" tramp-current-method ":" )))
		     (if tramp-current-user
			 (setq history-file (concat history-file tramp-current-user "@")))
		     (setq history-file (concat history-file tramp-current-host ":" ".bash_history"))
		     (setq comint-input-ring-file-name history-file)
		     (comint-read-input-ring t))))))

(custom-set-variables
 ;; custom-set-variables was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(Man-switches
   "-a -M /usr/local/man:/usr/local/share/man:/usr/share/man:/usr/X11/share/man:/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.11.sdk/usr/share/man")
 '(comint-input-ring-size 50000)
 '(dired-use-ls-dired nil)
 '(eval-expression-print-length nil)
 '(inhibit-startup-buffer-menu t)
 '(inhibit-startup-screen nil)
 '(initial-frame-alist (quote ((height . 50) (width . 100))))
 '(initial-scratch-message nil)
 '(ns-alternate-modifier (quote alt))
 '(ns-command-modifier (quote meta))
 '(read-quoted-char-radix 16)
 '(remote-shell-program "/usr/local/bin/ssh")
 '(shell-prompt-pattern "^[^:#$%>
]*[:#$%>] *")
 '(switch-to-visible-buffer nil)
 '(tramp-copy-size-limit 102400 nil (tramp))
 '(tramp-default-method "scp" nil (tramp))
 '(truncate-lines t))
(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(dired-flagged ((t (:inherit font-lock-warning-face :foreground "red")))))
